/*
 * Copyright 2019 Chocohead
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
package com.chocohead.ccc.api;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import net.minecraft.init.Blocks;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.tileentity.TileEntityChest;
import net.minecraft.util.WeightedRandomChestContent;
import net.minecraft.world.gen.feature.WorldGenDungeons;

import net.minecraftforge.common.ChestGenHooks;

public class CarefullyCastChestContentFillers {
	/**
	 * Callback for when {@link WorldGenDungeons} tries to fill a placed {@link Blocks#chest} that {@link TileEntity} doesn't extend {@link TileEntityChest} 
	 * 
	 * @author Chocohead
	 */
	public interface CarefullyCastChestContentFiller {
		/**
		 * Notification when the given (non-null) {@link TileEntity} would have been filled with loot if it was a {@link TileEntityChest}
		 * 
		 * <p>
		 * Note the default (vanilla) implementation is as follows:<pre>
		 * {@link WeightedRandomChestContent#generateChestContents}(rand, {@link ChestGenHooks#getItems}({@link ChestGenHooks#DUNGEON_CHEST}, rand), tile, {@link ChestGenHooks#getCount}({@link ChestGenHooks#DUNGEON_CHEST}, tile));
		 * </pre>
		 * 
		 * @param tile The {@link TileEntity} that would have been filled with dungeon loot
		 * @param rand The {@link Random} instance used by {@link WorldGenDungeons}
		 */
		void fillChest(TileEntity tile, Random rand);
	}
	private static final List<CarefullyCastChestContentFiller> FILLERS = new ArrayList<>();

	/**
	 * Register the given {@link CarefullyCastChestContentFiller} to be alerted during world generation
	 * 
	 * @param filler The {@link CarefullyCastChestContentFiller} to be registered
	 */
	public static void addFiller(CarefullyCastChestContentFiller filler) {
		if (filler == null) throw new IllegalArgumentException("Tried to add null CarefullyCastChestContentFiller");

		FILLERS.add(filler);
	}

	/**
	 * Run the registered {@link CarefullyCastChestContentFiller} for the given <b>non-null</b> {@link TileEntity}
	 * 
	 * @param tile The {@link TileEntity} that would've been filled with dungeon loot were it a standard {@link TileEntityChest vanilla chest}
	 * @param rand The {@link Random} instance used by {@link WorldGenDungeons}
	 */
	public static void fillChest(TileEntity tile, Random rand) {
		for (CarefullyCastChestContentFiller filler : FILLERS) {
			filler.fillChest(tile, rand);
		}
	}
}