/*
 * Copyright 2019 Chocohead
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
package com.chocohead.ccc.api;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import net.minecraft.init.Blocks;
import net.minecraft.tileentity.MobSpawnerBaseLogic;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.tileentity.TileEntityMobSpawner;
import net.minecraft.world.gen.feature.WorldGenDungeons;

public class CarefullyCastMobSpawnerSetters {
	/**
	 * Callback for when {@link WorldGenDungeons} tries to set a placed {@link Blocks#mob_spawner} that {@link TileEntity} doesn't extend {@link TileEntityMobSpawner} 
	 * 
	 * @author Chocohead
	 */
	public interface CarefullyCastMobSpawnerSetter {
		/**
		 * Notification when the given (non-null) {@link TileEntity} would have had the given mobName set if it was a {@link TileEntityMobSpawner}
		 * 
		 * <p>
		 * Note the default (vanilla) implementation is as follows:<pre>
		 * 
		 * tile.{@link TileEntityMobSpawner#func_145881_a() func_145881_a()}.{@link MobSpawnerBaseLogic#setEntityName}(mobName);
		 * </pre>
		 * 
		 * @param tile The {@link TileEntity} that would have had the mob type set
		 * @param mobName The name of the mob that the spawner was going to spawn, as given by {@link WorldGenDungeons#pickMobSpawner(Random)}
		 * 
		 * @return Whether this setter handled the spawner and thus no warning is needed about it being unset
		 */
		boolean handleSpawner(TileEntity tile, String mobName);
	}
	private static final List<CarefullyCastMobSpawnerSetter> SETTERS = new ArrayList<>();

	/**
	 * Register the given {@link CarefullyCastMobSpawnerSetter} to be alerted during world generation
	 * 
	 * @param filler The {@link CarefullyCastMobSpawnerSetter} to be registered
	 */
	public static void addFiller(CarefullyCastMobSpawnerSetter setter) {
		if (setter == null) throw new IllegalArgumentException("Tried to add null CarefullyCastMobSpawnerSetter");

		SETTERS.add(setter);
	}

	/**
	 * Run the registered {@link CarefullyCastChestContentFiller} for the given <b>non-null</b> {@link TileEntity}
	 * 
	 * @param tile The {@link TileEntity} that would've had the given mobName set if it was a {@link TileEntityMobSpawner vanilla spawner}
	 * @param mobName The name of the mob that was going to be set
	 * 
	 * @return Whether any of the registered {@link CarefullyCastMobSpawnerSetter}s handled setting the spawner
	 */
	public static boolean handleSpawner(TileEntity tile, String mobName) {
		boolean handled = false;

		for (CarefullyCastMobSpawnerSetter setter : SETTERS) {
			handled |= setter.handleSpawner(tile, mobName);
		}

		return handled;
	}
}